require 'dspace_rest_client'
require 'rainbow'

namespace :dspace do
  desc "Dspace tasks"

  task :import_items, [:log] => [:environment] do |t, args|
    desc "Import DSpace items to LearningObject Repository"

    include RepositoriesProxy

    args.with_defaults(:log => STDOUT)
    logger = Log::DatabaseLogger.new(args.log)
    logger.level = Log::DatabaseLogger::CREATE

    # Quantity of items fetched on each iteration
    limit = 500

    # Start point from where items will be fetched
    offset = 0

    loop do

      logger.info " --> importing items from #{offset} to #{offset+limit}"

      begin
        # Get items from dspace (from offset to offset+limit)
        dspace_items = get_items_from_dspace(limit,offset, ['metadata','bitstreams'])
      rescue => e
        logger.warn "Database error, going to sleep"
        logger.error e
        # Sleeps for a while to wait database's recovery
        sleep(10.seconds)
        # Goes to next iteration to retry
        next
      else
        # Terminate loop if there are no more items to import
        break if dspace_items.empty?

        # Increment offset, to get new items on next iteration
        offset = offset + limit

        # Iterates through items
        # Verifies if item is already on repository
        # Initializes new LearningObjects
        # and saves them on LearningObjects's repository
        dspace_items.each do |item|
          result = learning_object_repository.get_by_dspace_id item.id
          if result.nil?
            learning_object = initialize_learning_object item
            learning_object_repository.create learning_object
            # Comment the line below if you don't want print the itens
            logger.create "Imported: #{item.id} -> #{learning_object.id} "
          end
        end
      end
    end
    logger.close
  end

  task import_collections: :environment do
    desc "Import DSpace Collections to Institutions Repository"

    include RepositoriesProxy

    # Quantity of items fetched on each iteration
    limit = 500

    # Start point from where items will be fetched
    offset = 0

    loop do

      puts Rainbow(" --> importing Collections from #{offset} to #{offset+limit}").blue

      begin
        # Get items from dspace (from offset to offset+limit)
        dspace_items = get_items_from_dspace(limit,offset, ['parentCollection'])
      rescue
        # Sleeps for a while to wait database's recovery
        sleep(10.seconds)
        # Goes to next iteration to retry
        next
      else
        # Terminate loop if there are no more items to import
        break if dspace_items.empty?

        # Increment offset, to get new items on next iteration
        offset = offset + limit

        dspace_items.each do |item|

          # Checking if Item from DSpace exist in repository
          learning_object = learning_object_repository.get_by_dspace_id item.id
          
          unless learning_object.nil?

            if learning_object.publisher.nil? && !item.parent_collection.nil?

              # Get from repository the institution
              institution = institution_repository.find_by_name item.parent_collection.name

              if institution.nil?
                institution = initialize_institution item.parent_collection
                institution_repository.create institution
              end

              # Uncomment the line below if you want print the properties of itens
              # puts "\t[" + Rainbow(item.id).green.bright + "] " + Rainbow(item.name).yellow.bright + "[" + Rainbow(item.parent_collection.name).red +"]"

              # Setting publisher of learning_object as institution
              learning_object.publisher = institution
              learning_object_repository.create_relations(learning_object)

            end
          end
        end
      end
    end
  end

  private

  def dspace_repository
    @dspace_repository ||= Dspace::Client.instance.repository.item_repository
    
  end

  def get_items_from_dspace(limit, offset, expand=[])
    dspace_repository.get_all_items(
      expand: expand,
      limit: limit, offset: offset
    )
  end

  def initialize_institution item

    current_date = Time.now
    
    inst = Institution.new(
      :name => item.name,
      :last_modified => current_date,
      :created_at => current_date,
      :country => "",
      :city => "",
      :description => item.short_description
    )

  end

  def initialize_learning_object item
    metadata = build_array_of(item.metadata)
    bitstreams = build_array_of(item.bit_streams)
    current_date = Time.now

    lo = LearningObject.new(
      :name => item.name,
      :last_modified => current_date,
      :created_at => current_date,
      :id_dspace => item.id,
      :type => item.type,
      :thumbnail => nil,
      :bitstreams => bitstreams,
      :metadata => metadata
    )
    lo.author = lo.get_metadata_values_of("dc.contributor.author").join(', ')
    lo.description = lo.get_metadata_value_of("dc.description")
    date = lo.get_metadata_value_of("dc.date.issued")
    lo.published_at = Time.iso8601(date) unless date.nil?
    lo
  end

  def build_array_of(item_content=[])
    return item_content if item_content.nil?
    content_array = []
    item_content.each do |i|
      content_array << i.to_h
    end
    return content_array
  end

end
